@@ -104,7 +104,6 @@ def include_config(include, opts, orig_path, verbose):
     Parses extra configuration file(s) specified in an include list in the
     main config file.
     '''
-
     # Protect against empty option
     if not include:
         return opts
@@ -239,14 +238,14 @@ def minion_config(path):
     opts['open_mode'] = opts['open_mode'] is True
 
     # set up the extension_modules location from the cachedir
-    opts['extension_modules'] = os.path.join(opts['cachedir'], 'extmods')
+    opts['extension_modules'] = (
+            opts.get('extension_modules') or 
+            os.path.join(opts['cachedir'], 'extmods')
+            )
 
     # Prepend root_dir to other paths
     prepend_root_dir(opts, ['pki_dir', 'cachedir', 'log_file', 'sock_dir',
                             'key_logfile', 'extension_modules'])
-
-    opts['grains'] = salt.loader.grains(opts)
-
     return opts
 
 
@@ -274,20 +273,25 @@ def master_config(path):
             'pillar_roots': {
                 'base': ['/srv/pillar'],
                 },
-            'ext_pillar': {},
+            'ext_pillar': [],
+            # TODO - Set this to 2 by default in 0.10.5
+            'pillar_version': 1,
             'syndic_master': '',
             'runner_dirs': [],
             'client_acl': {},
+            'external_auth': {},
+            'token_expire': 720,
             'file_buffer_size': 1048576,
             'max_open_files': 100000,
             'hash_type': 'md5',
             'conf_file': path,
-            'pub_refresh': True,
+            'pub_refresh': False,
             'open_mode': False,
             'auto_accept': False,
             'renderer': 'yaml_jinja',
             'failhard': False,
             'state_top': 'top.sls',
+            'master_tops': {},
             'external_nodes': '',
             'order_masters': False,
             'job_cache': True,
@@ -327,10 +331,15 @@ def master_config(path):
 
     opts['aes'] = salt.crypt.Crypticle.generate_key_string()
 
-    opts['extension_modules'] = os.path.join(opts['cachedir'], 'extmods')
+    opts['extension_modules'] = (
+            opts.get('extension_modules') or 
+            os.path.join(opts['cachedir'], 'extmods')
+            )
+    opts['token_dir'] = os.path.join(opts['cachedir'], 'tokens')
     # Prepend root_dir to other paths
     prepend_root_dir(opts, ['pki_dir', 'cachedir', 'log_file',
-                            'sock_dir', 'key_logfile', 'extension_modules', 'autosign_file'])
+                            'sock_dir', 'key_logfile', 'extension_modules',
+                            'autosign_file', 'token_dir'])
 
     # Enabling open mode requires that the value be set to True, and
     # nothing else!
@@ -338,3 +347,21 @@ def master_config(path):
     opts['auto_accept'] = opts['auto_accept'] is True
     opts['file_roots'] = _validate_file_roots(opts['file_roots'])
     return opts
+
+
+def client_config(path):
+    '''
+    Load in the configuration data needed for the LocalClient. This function
+    searches for client specific configurations and adds them to the data from
+    the master configuration.
+    '''
+    opts = {'token_file': os.path.expanduser('~/.salt_token')}
+    opts.update(master_config(path))
+    cpath = os.path.expanduser('~/.salt')
+    load_config(opts, cpath, 'SALT_CLIENT_CONFIG')
+    if 'token_file' in opts:
+        opts['token_file'] = os.path.expanduser(opts['token_file'])
+    if os.path.isfile(opts['token_file']):
+        with open(opts['token_file']) as fp_:
+            opts['token'] = fp_.read().strip()
+    return opts
