@@ -24,7 +24,7 @@ def _detect_os():
     '''
     Apache commands and paths differ depending on packaging
     '''
-    httpd = ('CentOS', 'Scientific', 'RedHat', 'Fedora', 'Arch')
+    httpd = ('CentOS', 'Scientific', 'RedHat', 'Fedora', 'Arch', 'CloudLinux')
     apache2 = ('Ubuntu', 'Debian',)
     if __grains__['os'] in httpd:
         return 'apachectl'
