@@ -100,6 +100,16 @@ def restart(name):
     cmd = 'service {0} restart'.format(name)
     return not __salt__['cmd.retcode'](cmd)
 
+def reload(name):
+    '''
+    Reload the named service
+
+    CLI Example::
+
+        salt '*' service.reload <service name>
+    '''
+    cmd = 'service {0} reload'.format(name)
+    return not __salt__['cmd.retcode'](cmd)
 
 def status(name, sig=None):
     '''
