@@ -5,14 +5,10 @@ Publish a command from a minion to a target
 # Import python libs
 import ast
 
-# Import third party libs
-import zmq
-
 # Import salt libs
 import salt.crypt
 import salt.payload
-from salt._compat import string_types
-from salt.exceptions import SaltReqTimeoutError
+from salt._compat import string_types, integer_types
 
 def _publish(
         tgt,
@@ -39,20 +35,10 @@ def _publish(
 
         salt system.example.com publish.publish '*' cmd.run 'ls -la /tmp'
     '''
-    serial = salt.payload.Serial(__opts__)
     if fun == 'publish.publish':
         # Need to log something here
         return {}
-
-    if not arg:
-        arg = []
-
-    try:
-        if isinstance(ast.literal_eval(arg), dict):
-            arg = [arg,]
-    except Exception:
-        if isinstance(arg, string_types):
-            arg = arg.split(',')
+    arg = normalize_arg(arg)
 
     sreq = salt.payload.SREQ(__opts__['master_uri'])
     auth = salt.crypt.SAuth(__opts__)
@@ -71,6 +57,19 @@ def _publish(
     return auth.crypticle.loads(
             sreq.send('aes', auth.crypticle.dumps(load), 1))
 
+def normalize_arg(arg):
+    if not arg:
+        arg = []
+
+    try:
+        # Numeric checks here because of all numeric strings, like JIDs
+        if isinstance(ast.literal_eval(arg), (dict,integer_types,long)):
+            arg = [arg,]
+    except Exception:
+        if isinstance(arg, string_types):
+            arg = arg.split(',')
+
+    return arg
 
 def publish(tgt, fun, arg=None, expr_form='glob', returner='', timeout=5):
     '''
@@ -114,18 +113,9 @@ def runner(fun, arg=None):
 
         salt publish.runner manage.down
     '''
-    serial = salt.payload.Serial(__opts__)
-    if not arg:
-        arg = []
+    arg = normalize_arg(arg)
 
-    try:
-        if isinstance(ast.literal_eval(arg), dict):
-            arg = [arg,]
-    except Exception:
-        if isinstance(arg, string_types):
-            arg = arg.split(',')
-
-    sreq = salt.payload(__opts__['master_uri'])
+    sreq = salt.payload.SREQ(__opts__['master_uri'])
     auth = salt.crypt.SAuth(__opts__)
     tok = auth.gen_token('salt')
     load = {
