@@ -9,13 +9,13 @@ def __virtual__():
     return 'group' if __grains__['kernel'] == 'Windows' else False
 
 
-def add(name, gid=None):
+def add(name, gid=None, system=False):
     '''
     Add the specified group
 
     CLI Example::
 
-        salt '*' group.add foo 
+        salt '*' group.add foo
     '''
     cmd = 'net localgroup {0} /add'.format(name)
 
@@ -56,7 +56,7 @@ def info(name):
         if 'successfully' in line:
             memberline = False
         if memberline:
-            gr_mem.append(line.strip())    
+            gr_mem.append(line.strip())
         if '---' in line:
             memberline = True
     if not gr_name:
@@ -84,7 +84,7 @@ def getent():
         if 'successfully' in line:
             groupline = False
         if groupline:
-            ret.append(line.strip('*').strip())    
+            ret.append(line.strip('*').strip())
         if '---' in line:
             groupline = True
     for item in ret:
@@ -96,10 +96,10 @@ def getent():
             if 'successfully' in line:
                 memberline = False
             if memberline:
-                members.append(line.strip('*').strip())    
+                members.append(line.strip('*').strip())
             if '---' in line:
                 memberline = True
-        group = {'gid': gid, 
+        group = {'gid': gid,
                 'members': members,
                 'name': item,
                 'passwd': 'x'}
