@@ -52,11 +52,13 @@ class RemotePillar(object):
         load = {'id': self.id_,
                 'grains': self.grains,
                 'env': self.opts['environment'],
+                'ver': '2',
                 'cmd': '_pillar'}
-        return self.auth.crypticle.loads(
-                self.sreq.send('aes', self.auth.crypticle.dumps(load), 3, 7200)
-                )
-
+        ret = self.sreq.send('aes', self.auth.crypticle.dumps(load), 3, 7200)
+        key = self.auth.get_keys()
+        aes = key.private_decrypt(ret['key'], 4)
+        pcrypt = salt.crypt.Crypticle(self.opts, aes)
+        return pcrypt.loads(ret['pillar'])
 
 
 class Pillar(object):
@@ -67,16 +69,19 @@ class Pillar(object):
         # use the local file client
         self.opts = self.__gen_opts(opts, grains, id_, env)
         self.client = salt.fileclient.get_file_client(self.opts)
-        self.matcher = salt.minion.Matcher(self.opts)
-        self.functions = salt.loader.minion_mods(self.opts)
+        if opts.get('file_client', '') == 'local':
+            self.functions = salt.loader.minion_mods(opts)
+        else:
+            self.functions = salt.loader.minion_mods(self.opts)
+        self.matcher = salt.minion.Matcher(self.opts, self.functions)
         self.rend = salt.loader.render(self.opts, self.functions)
         self.ext_pillars = salt.loader.pillars(self.opts, self.functions)
 
-    def __gen_opts(self, opts, grains, id_, env=None):
+    def __gen_opts(self, opts_in, grains, id_, env=None):
         '''
         The options need to be altered to conform to the file client
         '''
-        opts = copy.deepcopy(opts)
+        opts = dict(opts_in)
         opts['file_roots'] = opts['pillar_roots']
         opts['file_client'] = 'local'
         opts['grains'] = grains
@@ -327,7 +332,7 @@ class Pillar(object):
                     else:
                         ext.update(self.ext_pillars[key](val))
                 except Exception:
-                    log.critical('Failed to load ext_pillar {0}'.format(key))
+                    log.exception('Failed to load ext_pillar {0}'.format(key))
         return ext
 
     def compile_pillar(self):
@@ -339,6 +344,8 @@ class Pillar(object):
         pillar, errors = self.render_pillar(matches)
         pillar.update(self.ext_pillar())
         errors.extend(terrors)
+        if self.opts.get('pillar_opts', False):
+            pillar['master'] = self.opts
         if errors:
             for error in errors:
                 log.critical('Pillar render error: {0}'.format(error))
