@@ -32,5 +32,5 @@ def render(template_file, env='', sls=''):
             sls=sls)
     if not tmp_data.get('result', False):
         raise SaltRenderError(tmp_data.get('data',
-            'Unknown render error in yaml_jinja renderer'))
+            'Unknown render error in json_jinja renderer'))
     return json.loads(tmp_data['data'])
