@@ -34,12 +34,15 @@ class CustomeConstructor(yaml.constructor.SafeConstructor):
             raise ConstructorError(None, None,
                     'expected a mapping node, but found {0}'.format(node.id),
                     node.start_mark)
+
+        self.flatten_mapping(node)
+
         mapping = {}
         for key_node, value_node in node.value:
             key = self.construct_object(key_node, deep=deep)
             try:
                 hash(key)
-            except TypeError as exc:
+            except TypeError:
                 err = ('While constructing a mapping {0} found unacceptable '
                        'key {1}').format(node.start_mark, key_node.start_mark)
                 raise ConstructorError(err)
