@@ -1,5 +1,7 @@
 # Import python libs
+import re
 import sys
+import yaml
 
 # Import salt libs
 from saltunittest import TestLoader, TextTestRunner
@@ -108,6 +110,31 @@ class MatchTest(integration.ShellCase, integration.ShellCaseCommonTestsMixIn):
         self.assertIn('sub_minion', data)
         self.assertNotIn('minion', data.replace('sub_minion', 'stub'))
 
+    def test_exsel(self):
+        data = self.run_salt('-X test.ping test.ping')
+        data = '\n'.join(data)
+        self.assertIn('minion', data)
+        self.assertIn('sub_minion', data)
+
+    def test_ipcadr(self):
+        subnets_data = self.run_salt('\'*\' network.subnets')
+        subnet = None
+        for line in re.findall(r'{[^}]+}', '\n'.join(subnets_data)):
+            try:
+                yamline = yaml.load(line)
+                subnet = yamline.values()[0][0]
+                break
+            except:
+                continue
+
+        if subnet is None:
+            self.skipTest('Failed to query the minion\'s subnets')
+
+        data = self.run_salt('-S {0} test.ping'.format(subnet))
+        data = '\n'.join(data)
+        self.assertIn('minion', data)
+        self.assertIn('sub_minion', data)
+
     def test_static(self):
         '''
         test salt static call
@@ -124,7 +151,6 @@ class MatchTest(integration.ShellCase, integration.ShellCaseCommonTestsMixIn):
         self.assertIn('user.add:', data)
 
 
-
 if __name__ == "__main__":
     loader = TestLoader()
     tests = loader.loadTestsFromTestCase(MatchTest)
