@@ -15,33 +15,35 @@ class SupervisordTest(integration.ModuleCase):
         if not ret:
             self.skipTest('supervisor not installed')
         if os.geteuid() != 0:
-            self.skipTest('you must be this root to run this test')
+            self.skipTest('You must be this root to run this test')
 
     def test_start(self):
         '''
         supervisord.running restart = False
         '''
-        ret = self.run_state('supervisord.running', name='null_service')
+        ret = self.run_state(
+            'supervisord.running', name='null_service', test=True)
 
         self.assertTrue(ret)
-        self.assertEqual(ret.items()[0][1]['result'], False)
+        self.assertEqual(self.state_result(ret), False)
 
     def test_restart(self):
         '''
         supervisord.running restart = True
         '''
         ret = self.run_state(
-            'supervisord.running', name='null_service', restart=True)
+            'supervisord.running', name='null_service', restart=True,
+            test=True)
 
         self.assertTrue(ret)
-        self.assertEqual(ret.items()[0][1]['result'], False)
+        self.assertEqual(self.state_result(ret), False)
 
     def test_stop(self):
         '''
         supervisord.dead
         '''
         ret = self.run_state(
-            'supervisord.dead', name='null_service')
+            'supervisord.dead', name='null_service', test=True)
 
         self.assertTrue(ret)
-        self.assertEqual(ret.items()[0][1]['result'], False)
+        self.assertEqual(self.state_result(ret), False)
